@@ -12,6 +12,8 @@ import SimpleITK as sitk
 from scipy import ndimage
 import cv2
 from stardist.models import StarDist2D
+from napari_animation import AnimationWidget
+from PIL import Image, ImageDraw
 
 # import skimage.io
 # from stardist.models import StarDist3D
@@ -19,7 +21,7 @@ from csbdeep.utils import normalize
 #from naparimovie import Movie
 import math
 import gc
-
+import json
 
 class NapariSTPT:
 
@@ -63,6 +65,7 @@ class NapariSTPT:
         self.crop_size_ratio_y = None
 
         self.crop = False
+        self.old_method = True
         
     def CropOriginal(self, volume):
 
@@ -123,29 +126,54 @@ class NapariSTPT:
             print("none-consolidated")
             self.ds1 = xr.open_zarr(self.search_folder.text() +
                                     str(self.comboBoxPath.currentText())+'/mos.zarr')
+
         # ds2 = xr.open_zarr(str(self.comboBoxPath.currentText())+'/mos.zarr', group='l.2')
         # ds4 = xr.open_zarr(str(self.comboBoxPath.currentText())+'/mos.zarr', group='l.4')
         # ds8 = xr.open_zarr(str(self.comboBoxPath.currentText())+'/mos.zarr', group='l.8')
         # ds32 = xr.open_zarr(str(self.comboBoxPath.currentText())+'/mos.zarr', group='l.32')
         
-        
+        # print("self.ds1.attrs")
+        # print(self.ds1.attrs)
+        # print("self.ds1.keys()")
+        # print(self.ds1.keys()["Coordinates"])
+        # print("self.ds1.attrs.keys()")
+        # print(self.ds1.attrs.keys())
+        # print("self.ds1['S002'].attrs")
+        # print(self.ds1['S002'].attrs)
+        # print(self.ds1[f"S{slice_no:03d}"].attrs)
 
-        self.slice_names = self.ds1.attrs['cube_reg']['slice']
+
+        #print(self.ds1['S002'].attrs['offsets']['x'])
+        # print(self.ds1['S002'].attrs['bscale'])
+        # print(json.loads(self.ds1['S002'].attrs['scale'])["x"])
+        # print(json.loads(self.ds1['S002'].attrs['scale'])["y"])
+        # print(json.loads(self.ds1['S002'].attrs['scale'])["z"])
+
+        #print(type(self.ds1['S002'].attrs['scale']))
+        #print(self.ds1['S002'].attrs['scale']["y"])
+        #print(self.ds1['S002'].attrs['scale']['z'])
+        #print(self.ds1['S001'].attrs['raw_meta'][0][0]['zres'])
+        #print(self.ds1['S001'].attrs.keys())
+        #print(self.ds1.attrs["multiscale"]['datasets'][1]['level'])
+        #print(self.ds1.attrs["multiscale"]['datasets'][1]['path'])
         # print(self.slice_names)
 
-        bscale = self.ds1.attrs['bscale']
-        bzero = self.ds1.attrs['bzero']
-        
-        #print(self.ds1['S001'].attrs['raw_meta'][0][0]['zres'])
-        #print(self.ds1.keys())
-        #print(self.ds1.attrs.keys())
-        #print(self.ds1['S001'].attrs.keys())
-        # print(self.ds1.attrs["multiscale"]['datasets'][1]['level'])
-        # print(self.ds1.attrs["multiscale"]['datasets'][1]['path'])
+        #old_method = True
 
-        self.align_x = self.ds1.attrs['cube_reg']['abs_dx']
-        self.align_y = self.ds1.attrs['cube_reg']['abs_dy']
-        print(f"self.align_x length {len(self.align_x)}")
+        if self.old_method:
+            #Old method
+            bscale = self.ds1.attrs['bscale']
+            bzero = self.ds1.attrs['bzero']
+            self.slice_names = self.ds1.attrs['cube_reg']['slice']
+            self.align_x = self.ds1.attrs['cube_reg']['abs_dx']
+            self.align_y = self.ds1.attrs['cube_reg']['abs_dy']
+            print(f"self.align_x length {len(self.align_x)}")
+            print(f"self.align_x {self.align_x}")
+            print(f"self.align_y {self.align_y}")
+        else:
+            
+            bscale = self.ds1['S001'].attrs['bscale']
+            bzero = self.ds1['S001'].attrs['bzero']
 
         # print("size: {}".format(self.ds1.dims))
 
@@ -169,13 +197,19 @@ class NapariSTPT:
             resolution = 1
             index = 0
         
-        print("loading at resolution {}".format(resolution))
-        #index = self.comboBoxResolution.currentIndex()
-        print("group: " +
-              self.ds1.attrs["multiscale"]['datasets'][index]['path'])
-        gr = self.ds1.attrs["multiscale"]['datasets'][index]['path']
-        ds = xr.open_zarr(self.search_folder.text(
-        ) + str(self.comboBoxPath.currentText())+'/mos.zarr', group=gr)
+        if self.old_method:
+            print("loading at resolution {}".format(resolution))
+            #index = self.comboBoxResolution.currentIndex()
+            print("group: " + self.ds1.attrs["multiscale"]['datasets'][index]['path'])
+            gr = self.ds1.attrs["multiscale"]['datasets'][index]['path']
+            ds = xr.open_zarr(self.search_folder.text() + str(self.comboBoxPath.currentText())+'/mos.zarr', group=gr)
+        else:
+            print("resolution")
+            print(resolution)
+            print(json.loads(self.ds1.attrs["multiscale"])['datasets'][index]['path'])
+            gr = json.loads(self.ds1.attrs["multiscale"])['datasets'][index]['path']
+            print(gr)
+            ds = xr.open_zarr(self.search_folder.text() + str(self.comboBoxPath.currentText())+'/mos.zarr', group=gr)
 
         self.optical_slices = len(ds.z)
         print(f"optical_slices available: {self.optical_slices}")
@@ -227,8 +261,8 @@ class NapariSTPT:
                 # self.aligned_1 = volume_1
 
                 print("aligning C1")
-                self.aligned_1 = self.Align(
-                    volume_1, resolution, output_resolution)
+                self.aligned_1 = self.Align(volume_1, resolution, output_resolution)
+                
                 self.shape = self.aligned_1.shape
                 self.viewer.add_image([self.aligned_1], name='C1', scale=(
                     15/self.optical_slices, output_resolution, output_resolution), blending='additive', colormap='bop purple', translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
@@ -522,6 +556,7 @@ class NapariSTPT:
                 self.viewer.add_image([im1, im2, im4, im8, im16, im32], multiscale=True,
                                       name='C4', blending='additive', colormap='bop blue')
 
+
     def Align(self, volume, resolution, output_resolution):
 
         spacing = (self.ds1['S001'].attrs['scale'])
@@ -534,8 +569,8 @@ class NapariSTPT:
         size2D = (
             int(size_multiplier*volume.shape[2]), int(size_multiplier*volume.shape[1]))
 
-        #print("spacing: {}".format(spacing))
-        #print("volume.shape[0]: {}".format(volume.shape[0]))
+        print("spacing: {}".format(spacing))
+        print("volume.shape[0]: {}".format(volume.shape[0]))
 
         z_size = volume.shape[0]
         for z in range(0, z_size):
@@ -551,7 +586,7 @@ class NapariSTPT:
             slice_name = self.slice_names[z]
 
             spacing = (self.ds1[slice_name].attrs['scale'])
-            #print(f"spacing {spacing}")
+            print(f"spacing {spacing}")
             # fixed.SetSpacing([1.6*spacing[0],1.6*spacing[1]])
             fixed.SetSpacing([resolution*0.1*spacing[1],
                              resolution*0.1*spacing[0]])
@@ -573,6 +608,8 @@ class NapariSTPT:
             # transform.SetTranslation([0,0])
             print('{}/{}, translate_x: {}, translate_y: {}'.format(z,
                   z_size, alignX, alignY))
+            print('{}/{}, align_x[z]: {}, align_y[z]: {}'.format(z,
+                  z_size, self.align_x[z], self.align_y[z]))
 
             resampler = sitk.ResampleImageFilter()
             # resampler.SetReferenceImage(fixed)
@@ -591,6 +628,108 @@ class NapariSTPT:
 
         return aligned.astype(dtype=np.float32)
 
+    def AlignNew(self, volume, resolution, output_resolution):
+        
+        if self.old_method:
+            spacing = (self.ds1['S001'].attrs['scale'])
+        else:
+            spacing = [float,float,float]
+            #print(self.ds1['S001'].attrs['scale'])
+            #print(json.loads(self.ds1['S001'].attrs['scale']))
+            #print(json.loads(self.ds1['S001'].attrs['scale'])["x"])
+            #print(type(json.loads(self.ds1['S001'].attrs['scale'])["x"]))
+            spacing[0] = float(json.loads(self.ds1['S001'].attrs['scale'])["x"])
+            spacing[1] = float(json.loads(self.ds1['S001'].attrs['scale'])["y"])
+            spacing[2] = float(json.loads(self.ds1['S001'].attrs['scale'])["z"])
+            print(spacing[0])
+            print(spacing[1])
+            print(spacing[2])
+
+        # size_multiplier = (resolution*0.1*spacing[0])/7.5
+        # print(resolution*0.1*spacing[0])
+        size_multiplier = (resolution*spacing[0])/output_resolution
+        size = (volume.shape[0], int(size_multiplier*volume.shape[1]), int(size_multiplier*volume.shape[2]))
+        aligned = np.zeros(size, dtype=np.float32)
+        size2D = (int(size_multiplier*volume.shape[2]), int(size_multiplier*volume.shape[1]))
+
+        #print("spacing: {}".format(spacing))
+        #print("volume.shape[0]: {}".format(volume.shape[0]))
+
+        z_size = volume.shape[0]
+        for z in range(1, z_size+1):
+
+            fixed = sitk.GetImageFromArray(volume[z-1, :, :])
+            fixed.SetOrigin((0, 0))
+
+            # slice_name = 'S00'+str(z+1)
+            # if(z+1 > 9):
+            #    slice_name = 'S0'+str(z+1)
+            # if(z+1 > 99):
+            #    slice_name = 'S'+str(z+1)
+            if self.old_method:
+                slice_name = self.slice_names[z-1]
+                print(slice_name)
+            else:
+                slice_name = f"S{z:03d}"
+
+                spacing[0] = float(json.loads(self.ds1[slice_name].attrs['scale'])["x"])
+                spacing[1] = float(json.loads(self.ds1[slice_name].attrs['scale'])["y"])
+                spacing[2] = float(json.loads(self.ds1[slice_name].attrs['scale'])["z"])
+
+            #print(spacing)
+            #spacing = (self.ds1[slice_name].attrs['scale'])
+            #print(f"spacing {spacing}")
+            # fixed.SetSpacing([1.6*spacing[0],1.6*spacing[1]])
+            #print(type(resolution))
+            #print(type(spacing[1]))
+            
+            fixed.SetSpacing([resolution*spacing[1],
+                             resolution*spacing[0]])
+
+            transform = sitk.Euler2DTransform()
+
+            if self.old_method:
+                alignY = 0
+                if not np.isnan(self.align_y[z-1]):
+                    alignY = -self.align_y[z-1]*0.1*spacing[1]
+
+                alignX = 0
+                if not np.isnan(self.align_x[z-1]):
+                    alignX = -self.align_x[z-1]*0.1*spacing[0]
+                print(alignY)
+            else:
+                alignX = self.ds1[slice_name].attrs['offsets']['x']*spacing[0]
+                alignY = self.ds1[slice_name].attrs['offsets']['y']*spacing[1]
+                print('{}/{}, offsets_x: {}, offsets_y: {}'.format(z, z_size, self.ds1[slice_name].attrs['offsets']['x'], self.ds1[slice_name].attrs['offsets']['y']))
+                # print('{}/{}, spacing[0]: {}, spacing[1]: {}'.format(z, z_size, spacing[0], spacing[1]))
+                
+                #alignX = 0
+                #alignY = 0
+
+            transform.SetTranslation([alignY, alignX])
+
+            # transform.SetTranslation([-align_y[z]*0.1*spacing[1],-align_x[z]*0.1*spacing[0]])
+            # transform.SetTranslation([0.5*-align_y[z],0.5*-align_x[z]])
+            # transform.SetTranslation([0,0])
+            # print('{}/{}, alignX: {}, alignY: {}'.format(z, z_size, alignX, alignY))
+
+            resampler = sitk.ResampleImageFilter()
+            # resampler.SetReferenceImage(fixed)
+
+            resampler.SetSize(size2D)
+            resampler.SetOutputSpacing([output_resolution, output_resolution])
+            resampler.SetOutputOrigin((0, 0))
+            resampler.SetInterpolator(sitk.sitkLinear)
+            resampler.SetDefaultPixelValue(0)
+            resampler.SetTransform(transform)
+
+            out = resampler.Execute(fixed)
+
+            np_out = sitk.GetArrayFromImage(out)
+            aligned[z-1, :, :] = np_out
+
+        return aligned.astype(dtype=np.float32)
+
     def Remove_Regions(self, use_size):
 
         # output_resolution = float(self.pixel_size.text())
@@ -914,6 +1053,227 @@ class NapariSTPT:
             
         self.viewer.layers.remove('Shapes')
 
+                
+    def add_polygon_simple(self):
+
+        pos = int(self.viewer.cursor.position[0]/15)
+
+        data_length = len(self.viewer.layers['Shapes'].data[0])
+        print(data_length)
+
+        new_shape = []
+        for i in range(0, data_length):
+            
+            x = pos
+            y = self.viewer.layers['Shapes'].data[0][i][1]
+            z = self.viewer.layers['Shapes'].data[0][i][2]
+            new_shape.append((x,y,z))
+
+        new_shape = np.array(new_shape)
+        
+        shapes_layer = self.viewer.add_shapes(new_shape, shape_type='polygon', name = "Shapes", scale=(15, 15, 15),)
+    
+    
+    def add_polygon(self):
+
+        pos = int(self.viewer.cursor.position[0]/15)
+        
+        data_length = len(self.viewer.layers['Shapes'].data[0])
+        print(data_length)
+
+        contour_list = []
+        z_pos = self.viewer.layers["Shapes"].data[0][0][0] # z pos
+        contour = self.viewer.layers["Shapes"].data[0]
+        contour_list.append([z_pos,contour])
+
+        i = 1
+        while True:
+            layer_name = "Shapes [{}]".format(i)
+            try:
+                z_pos = self.viewer.layers[layer_name].data[0][0][0] # z pos
+                contour = self.viewer.layers[layer_name].data[0]
+                contour_list.append((z_pos,contour))
+                i = i+1
+            except:
+                break
+        contour_list_sorted = sorted(contour_list, key=lambda tup: tup[0])
+
+
+
+
+
+
+        
+            
+        new_shape = []
+
+        if pos < contour_list_sorted[0][0]:
+            for i in range(0, data_length):
+                x = pos
+                y = float(contour_list_sorted[0][1][i][1])
+                z = float(contour_list_sorted[0][1][i][2])
+
+                new_shape.append((x,y,z))
+
+        elif pos > contour_list_sorted[len(contour_list_sorted)-1][0]:
+            for i in range(0, data_length):
+                x = pos
+                y = float(contour_list_sorted[len(contour_list_sorted)-1][1][i][1])
+                z = float(contour_list_sorted[len(contour_list_sorted)-1][1][i][2])
+
+                new_shape.append((x,y,z))
+            
+        else:
+            z_i = 0
+            for i in range(0,len(contour_list_sorted)-1):
+                z_level_start = contour_list_sorted[i][0]
+                z_level_end = contour_list_sorted[i+1][0]
+                if pos >= z_level_start and pos <= z_level_end:
+                    z_i = i
+
+            for i in range(0, data_length):
+                x1 = float(contour_list_sorted[z_i][1][i][0])
+                y1 = float(contour_list_sorted[z_i][1][i][1])
+                z1 = float(contour_list_sorted[z_i][1][i][2])
+
+                x2 = float(contour_list_sorted[z_i+1][1][i][0])
+                y2 = float(contour_list_sorted[z_i+1][1][i][1])
+                z2 = float(contour_list_sorted[z_i+1][1][i][2])
+
+                weight2 = (pos - x1) / (x2 - x1)
+                weight = 1 - weight2
+
+                x = pos
+                y = (weight * y1) + (weight2 * y2)
+                z = (weight * z1) + (weight2 * z2)
+                new_shape.append((x,y,z))
+
+
+        new_shape = np.array(new_shape)
+        shapes_layer = self.viewer.add_shapes(new_shape, shape_type='polygon', name = "Shapes", scale=(15, 15, 15),)
+    
+
+
+
+
+    def run_remove_outside(self):
+
+        
+
+        
+        if self.aligned_1 is not None:
+            aligned1_tmp = np.copy(self.aligned_1)
+        if self.aligned_2 is not None:
+            aligned2_tmp = np.copy(self.aligned_2)
+        if self.aligned_3 is not None:
+            aligned3_tmp = np.copy(self.aligned_3)
+        if self.aligned_4 is not None:
+            aligned4_tmp = np.copy(self.aligned_4)
+
+        contour_list = []
+        z_pos = self.viewer.layers["Shapes"].data[0][0][0] # z pos
+        contour = self.viewer.layers["Shapes"].data[0]
+        contour_list.append([z_pos,contour])
+
+
+        i = 1
+        while True:
+            layer_name = "Shapes [{}]".format(i)
+            try:
+                z_pos = self.viewer.layers[layer_name].data[0][0][0] # z pos
+                contour = self.viewer.layers[layer_name].data[0]
+                contour_list.append((z_pos,contour))
+                i = i+1
+            except:
+                break
+        contour_list_sorted = sorted(contour_list, key=lambda tup: tup[0])
+        
+
+        
+
+        output_resolution = float(self.pixel_size.text())
+        data_length = len(self.viewer.layers['Shapes'].data[0])
+        print("data_length {}".format(data_length))
+
+        c = 0
+        width = 0
+        height = 0
+        if self.aligned_1 is not None:
+            c, width, height = self.aligned_1.shape
+        if self.aligned_2 is not None:
+            c, width, height = self.aligned_2.shape
+        if self.aligned_3 is not None:
+            c, width, height = self.aligned_3.shape
+        if self.aligned_4 is not None:
+            c, width, height = self.aligned_4.shape
+
+        for z_level in range(0,c):
+
+            z_i = -1
+            for i in range(0,len(contour_list_sorted)-1):
+                z_level_start = contour_list_sorted[i][0]
+                z_level_end = contour_list_sorted[i+1][0]
+                if z_level >= z_level_start and z_level <= z_level_end:
+                    z_i = i
+
+            if z_i == -1:
+                mask = np.zeros((width, height), dtype=int)
+            else:
+                polygon_values = []
+                for i in range(0, data_length):
+                    x1 = float(contour_list_sorted[z_i][1][i][0])
+                    y1 = float(contour_list_sorted[z_i][1][i][1])
+                    z1 = float(contour_list_sorted[z_i][1][i][2])
+
+                    x2 = float(contour_list_sorted[z_i+1][1][i][0])
+                    y2 = float(contour_list_sorted[z_i+1][1][i][1])
+                    z2 = float(contour_list_sorted[z_i+1][1][i][2])
+
+                    weight2 = (z_level - x1) / (x2 - x1)
+                    weight = 1 - weight2
+
+                    y = (weight * y1) + (weight2 * y2)
+                    z = (weight * z1) + (weight2 * z2)
+                    polygon_values.append((y, z))
+
+
+                img = Image.new('L', (width, height), 0)
+                ImageDraw.Draw(img).polygon(polygon_values, outline=1, fill=1)
+
+                mask = np.array(img)
+                mask = np.transpose(mask, (1,0))
+
+                #self.viewer.add_image([mask], name='mask', scale=(output_resolution, output_resolution), blending='additive', colormap='bop purple')
+                #aligned3_tmp[z_level,:,:] = aligned3_tmp[z_level,:,:] * mask
+
+            
+            if self.aligned_1 is not None:
+                aligned1_tmp[z_level,:,:] = aligned1_tmp[z_level,:,:] * mask
+            if self.aligned_2 is not None:
+                aligned2_tmp[z_level,:,:] = aligned2_tmp[z_level,:,:] * mask
+            if self.aligned_3 is not None:
+                aligned3_tmp[z_level,:,:] = aligned3_tmp[z_level,:,:] * mask
+            if self.aligned_4 is not None:
+                aligned4_tmp[z_level,:,:] = aligned4_tmp[z_level,:,:] * mask
+
+        
+        #self.viewer.layers.remove('C3')
+           
+
+
+        if self.aligned_1 is not None:
+            self.viewer.add_image([aligned1_tmp], name='C1_masked', scale=(15, output_resolution, output_resolution), blending='additive', colormap='bop purple', translate=(0, 0, 0))
+        if self.aligned_2 is not None:
+            self.viewer.add_image([aligned2_tmp], name='C2_masked', scale=(15, output_resolution, output_resolution), blending='additive', colormap='red', translate=(0, 0, 0))
+        if self.aligned_3 is not None:
+            self.viewer.add_image([aligned3_tmp], name='C3_masked', scale=(15, output_resolution, output_resolution), blending='additive', colormap='green', translate=(0, 0, 0))
+        if self.aligned_4 is not None:
+            self.viewer.add_image([aligned4_tmp], name='C4_masked', scale=(15, output_resolution, output_resolution), blending='additive', colormap='bop blue', translate=(0, 0, 0))
+
+
+        #self.viewer.layers.remove('Shapes')
+
+
     def SelectFolder(self):
         file = str(QtWidgets.QFileDialog.getExistingDirectory())
         file = file + '/'
@@ -1221,7 +1581,10 @@ class NapariSTPT:
             
             print(sys.platform)
             if sys.platform == 'linux':
-                self.search_folder = QtWidgets.QLineEdit('/data/meds1_c/storage/processed/stpt')
+                if self.old_method:
+                    self.search_folder = QtWidgets.QLineEdit('/data/meds1_c/storage/processed/stpt')
+                else:
+                    self.search_folder = QtWidgets.QLineEdit('/data/meds1_a/eglez/stpt_out')
             else:
                 self.search_folder = QtWidgets.QLineEdit('N:/stpt/')
             
@@ -1386,23 +1749,42 @@ class NapariSTPT:
             #bSaveMovie.setCheckable(True)
             #bSaveMovie.clicked.connect(self.SaveMovie)
             #hbox.addWidget(bSaveMovie)
+            hbox.addStretch(1)
+            vbox.addLayout(hbox)
+
+            #bStardist = QtWidgets.QPushButton('Run Stardist in shape')
+            #bStardist.setCheckable(True)
+            #bStardist.clicked.connect(self.run_stardist)
+            #hbox = QtWidgets.QHBoxLayout()
+            #hbox.addWidget(bStardist)
             #hbox.addStretch(1)
             #vbox.addLayout(hbox)
 
-            bStardist = QtWidgets.QPushButton('Run Stardist in shape')
-            bStardist.setCheckable(True)
-            bStardist.clicked.connect(self.run_stardist)
+            #vbox.addStretch(1)
+            
+            bAddPolygon = QtWidgets.QPushButton('Add')
+            bAddPolygon.setCheckable(True)
+            bAddPolygon.clicked.connect(self.add_polygon)
+            bRemoveOutside = QtWidgets.QPushButton('Remove outside interpolated region')
+            bRemoveOutside.setCheckable(True)
+            bRemoveOutside.clicked.connect(self.run_remove_outside)
             hbox = QtWidgets.QHBoxLayout()
-            hbox.addWidget(bStardist)
+            hbox.addWidget(bAddPolygon)
+            hbox.addWidget(bRemoveOutside)
             hbox.addStretch(1)
             vbox.addLayout(hbox)
 
             vbox.addStretch(1)
 
+
+
             
             widget.setLayout(vbox)
             self.viewer.window.add_dock_widget(widget, area="right")
 
+            animation_widget = AnimationWidget(self.viewer)
+            self.viewer.window.add_dock_widget(animation_widget, area='right')
+
             # inputfile = ''
             # try:
             #     opts, args = getopt.getopt(
